function [Bs, alphaEsts, tuningparams, rankEsts, k0, Nu_scl] = Func_TISPPath_Mat(X, Y, family, ...
    pathRangeGeneration, lambdasSpec, nPoints, thresholdingWay, errBnd, maxIT, Nu, intercptchoice, updating, relaxWay)
% Compute the matrix-TISP solution path. 
% Arguments: 
%    Parameter pathRangeGeneration currently can be 'value', 'number', 'specified'. 
%    The first is the common way. The 'number' option to generate the search grid 
%    is appropriate for quantile (or hard-number/hybrid-number) thresholding. 
%    If is set to be 'specified', lambdasSpec should be provided. 
%    Note that Nu is imposed on the coefficients for X. A scaled adjustment
%    is computed below (X_scl <- X/k0, Nu_scl <- Nu/k0^2)

if strcmp(pathRangeGeneration, 'value') & ( strcmp(lower(thresholdingWay), 'hybrid-number') | strcmp(lower(thresholdingWay), 'hard-number'))
    error('Thresholding rule and path generation manner do not match')
end
if strcmp(pathRangeGeneration, 'number') & ~( strcmp(lower(thresholdingWay), 'hybrid-number') | strcmp(lower(thresholdingWay), 'hard-number'))
    error('Thresholding rule and path generation manner do not match')
end

[n, d] = size(X); 
m = size(Y, 2);

% nPoints = 50 % 100; %500; %150 %   75;
nzUBnd = min(d, m);  
    
%%% scaling
switch lower(family)
   case {'binomial'}
        k0 = .5*norm(X, 2);
   case 'gaussian'
        k0 = norm(X, 2);  
   otherwise
      error('not implemented yet')
end  
X_scl = X ./ k0;
Nu_scl = Nu / k0^2; % get the ridge (l2, Frobenius) parameter for the scaled X
%%%%%%%%%


if strcmp(intercptchoice, 'none') | strcmp(intercptchoice, 'simuopt2')
    intercept = 0; %[];
elseif strcmp(intercptchoice, 'simuopt')
    intercept = 0; %[]; %GLM_canlink(mean(y), family) ; % a better est????
elseif strcmp(intercptchoice, 'fixedopt')
    intercept = ones(size(X_scl, 1), 1) * GLM_canlink(mean(Y, 1), family) ; 
end

BInit = zeros(size(X_scl, 2), size(Y, 2));

if strcmp(pathRangeGeneration, 'value')
    lamUBnd_1 = svd( BInit + X_scl'*Y - X_scl' * GLM_mean(X_scl, BInit, family, intercept) );
    lamUBnd = max( abs(lamUBnd_1) ); 
    lambdas = 0:lamUBnd/nPoints:(lamUBnd+lamUBnd/nPoints); % 1  %0.01:.005:2
    % lambdas = 3.^(-6:(log(lamUBnd)/log(3)+6)/nPoints:log(lamUBnd+lamUBnd/nPoints)/log(3));  %exp(-10:(log(lamUBnd)+10)/nPoints:log(lamUBnd+lamUBnd/nPoints)); 
    % lambdas = lambdas(1:nPoints); %%% TEMP
    lambdas = lambdas(end:-1:1);
elseif strcmp(pathRangeGeneration, 'number')
    lambdas = [1:nzUBnd];
elseif strcmp(pathRangeGeneration, 'specified')
    lambdas = reshape(lambdasSpec, 1, numel(lambdasSpec));
else
    disp('This pathRangeGeneration is not implemented yet..')
end


Bs = zeros(d, m, numel(lambdas));
rankEsts = zeros(1, numel(lambdas));
% if ~strcmp(intercptchoice, 'none')
    alphaEsts = -ones(m, numel(lambdas)); % record the intercept estimates (we only take the first row vector of each alpha matrix
% end
%%%%%%%% compute the solution path %%%%%
for lambdaInd = 1:numel(lambdas)
    fprintf('.'); 
    if mod(lambdaInd, 50) == 0
        fprintf('\n'); 
    end
    Lambda = lambdas(lambdaInd);
    if abs(Lambda) < 1e-6 % && ( strcmp(thresholdingWay, 'hard') || strcmp(thresholdingWay, 'hybrid') ) % == 0 %
        disp('lambda is 0 or close to 0')
        alphaEst = zeros(n, m);  % make it a matrix  
        BEst = zeros(d, m);
        % Run Newton's algorithm directly
        if strcmp(intercptchoice, 'none') | strcmp(intercptchoice, 'fixedopt') % with intercept (0 or fixed)
            if strcmp(intercptchoice, 'fixedopt')
                alphaEst = intercept*ones(n, m);           
            end
            for trespInd = 1:m
                BEst(:, trespInd) = GLM_opt(X_scl, Y(:, trespInd), Nu_scl, family, alphaEst(:, trespInd));
            end       
            sing = svd(BEst);
        elseif strcmp(intercptchoice, 'simuopt')
%             for trespInd = 1:m
%                 tmpw = GLM_opt([ones(n, 1), X_scl], Y(:, trespInd), [0; Nu_scl*ones(d,1)], family);
%                 BEst(:, trespInd) = tmpw(2:end);
%                 alphaEst(:, trespInd) = tmpw(1)*ones(n, 1);           
%             end
            tmpws = GLM_opt_Multi([ones(n, 1), X_scl], Y, [zeros(1, m);  Nu_scl*ones(d-1,m)], family) ;
            BEst = tmpws(2:end,:);
            alphaEst = ones(n, 1) *tmpws(1, :); 
            sing = svd(BEst);
        elseif strcmp(intercptchoice, 'simuopt2')
            BEst = GLM_opt_Multi(X_scl, Y, [zeros(1, m);  Nu_scl*ones(d-1,m)], family);
            alphaEst = intercept*ones(n, m);  % should be zero. The true intercept is the first row of BEst
            sing = svd(BEst(2:end,:));
        end
        
        % Newton's algorithm ends        
    else
        wts = Lambda; %*ones(size(Y, 2), 1); 
        [BEst, alphaEst, sing, totNum] = TISP_Mat(X_scl, Y, wts, family, thresholdingWay, errBnd, maxIT, BInit, Nu_scl, intercptchoice, intercept, updating, relaxWay);
    end
    
    Bs(:, :, lambdaInd) = BEst; 
    alphaEsts(:, lambdaInd) = alphaEst(1, :)'; % The alphaEst obtained in any way above is a matrix of the same size as Y
    rankEsts(lambdaInd) = sum(sing > 1e-4);
    
%     Binit = BEst; %% TEMP %% only for nuclear norm!!

    if rankEsts(lambdaInd) > nzUBnd
        matBetaEsts(:,:,lambdas<Lambda) = [];
        alphaEsts(:,lambdas<Lambda) = [];        
        lambdas(lambdas<Lambda) = []; 
        break; 
        disp('Early termination of the loop!')
    end
end
fprintf('\n'); 

%%% Scale back!!
Bs = Bs / k0; % No need for alpha 

% lambdas should also be scaled back !!!! -- no general formula
tuningparams = lambdas;


